@@ -1,6 +1,7 @@
 #!/usr/bin/env python3
 import sys, unittest
 
+import setup_test
 import libxml2
 
 class TestCase(unittest.TestCase):
@@ -15,7 +16,6 @@ class TestCase(unittest.TestCase):
     def tearDown(self):
         libxml2.cleanupParser()
         if libxml2.debugMemory(1) != 0:
-            libxml2.dumpMemory() 
             self.fail("Memory leak %d bytes" % (libxml2.debugMemory(1),))
         else:
             print("OK")
@@ -58,9 +58,9 @@ class TestCase(unittest.TestCase):
                         ("dummy.xml",None,0),
                         libxml2.treeError,
                         domain=libxml2.XML_FROM_IO,
-                        code=libxml2.XML_IO_LOAD_ERROR,
-                        message='failed to load external entity "dummy.xml"\n',
-                        level=libxml2.XML_ERR_WARNING,
+                        code=libxml2.XML_IO_ENOENT,
+                        message='failed to load "dummy.xml": No such file or directory\n',
+                        level=libxml2.XML_ERR_FATAL,
                         file=None,
                         line=0)
 
@@ -71,8 +71,8 @@ class TestCase(unittest.TestCase):
                         (s,len(s),"dummy.xml",None,0),
                         libxml2.treeError,
                         domain=libxml2.XML_FROM_PARSER,
-                        code=libxml2.XML_ERR_TAG_NOT_FINISHED,
-                        message='Premature end of data in tag x line 1\n',
+                        code=libxml2.XML_ERR_TAG_NAME_MISMATCH,
+                        message='Opening and ending tag mismatch: a line 2 and x\n',
                         level=libxml2.XML_ERR_FATAL,
                         file='dummy.xml',
                         line=3)
